#pragma once

#include "menuitem.h"
#include "unassignable.h"
#include "wimp_menu.h"
#include <string>
#include <list>

namespace rov2
{

class CMenu;
class CTask;

/*! Extends the tMenu struct with a special key which is used to identify it as a menu generated by ROVLib2 */
typedef struct
{
  /*! The magic key*/
  unsigned int uiKey;
  /*! The menu which generated this */
  CMenu* pmMenu;
  /*! The WIMP menu structure */
  tMenu mMenu;
} tCheckedMenu; // cm

/*! Class to handle WIMP menus*/
class CMenu : public CUnassignable
{
public:
  /*! Initialises the menu
  \param rksTitle The menu title (string is copied)
  \param iMaxItems The maximum number of items which are allowed in this menu */
  CMenu(const std::string& rksTitle, size_t iMaxItems);
  ~CMenu();

  /*! Adds a menu item to this menu
  \param pmiItem the item to add
  */
  void AddItem(CMenuItem* pmiItem);

  /*! Displays this menu as an icon bar*/
  void DisplayAsIconbarMenu();

  /*! Called when the menu is clicked on
  \param kuiaEntries The list of entries for this item onwards (-1 to end)
  \return How the menu was handled
  */
  virtual eMenuHandled Clicked(const unsigned int* kuiaEntries);

  /*! The user-defineable context for this menu */
  void* m_pContext;

  /*! Populates the menu structure
  \return The size for this menu
  */
  virtual tLocation PopulateMenu();

  /*! Gets the underlying menu structure
  \return The menu structure
  */
  inline const tCheckedMenu* GetMenu() { return m_mcMenu; }

protected:

private:
  /*! The list of menu items */
  std::list<CMenuItem*> m_lpmiItems;

  /*! The copy of the title */
  std::string m_sTitle;

  /*! The internal menu structure */
  tCheckedMenu* m_mcMenu;

  /*! The magic key */
  static unsigned int s_kuiKey;

  /*! The maximum number of items */
  size_t m_iMaxItems;
}; // m

}

